""" General PyQt Utilities
"""
import collections
from contextlib  import contextmanager
from PyQt4 import QtCore, QtGui

@contextmanager
def signalsBlocked(*args):
	""" Block signals with context manager
		Args: 
			args: widget or widgets
	"""
	states = [(target, target.signalsBlocked()) for target in flattenArgs(args)]
	try:
		for target, _ in states:
			target.blockSignals(True)
		yield
	finally:
		for target, blockedState in states:
			target.blockSignals(blockedState)


def showWaitCursor(f):
	""" Decorator for display a wait cursor whilst in a slow function
	"""
	def new_f(*args, **kwargs):
		QtGui.QApplication.setOverrideCursor( QtCore.Qt.WaitCursor )
		try:
			return f(*args, **kwargs)
		finally:
			QtGui.QApplication.restoreOverrideCursor()
	new_f.__name__ = f.__name__
	new_f.__doc__  = f.__doc__
	return new_f

def flattenArgs(items):
	""" Generator to expand a sequence of items recursively.
		Eg (1, [2, 3], ["a", "b", [10]]) -> (1, 2, 3, "a", "b", 10)
	"""
	for item in items:
		if isinstance(item, collections.Iterable) and not isinstance(item, basestring):
			for sub in flattenArgs(item):
				yield sub
		else:
			yield item


class _SignalTracker(QtCore.QObject):
	""" Base class for signal tracking and monitoring. Should not be instantiated directly
	"""
	def __init__(self, showTracer):
		super(_SignalTracker, self).__init__()
		self._showTracer = showTracer
		self._counter = 0

	def monitor(self, *widgets):
		""" Monitor the signals being emitted by one or more objects.
			Accepts single or multiple widgets, or sequences of widgets
		"""
		for widget in flattenArgs(widgets):
			for signal in self._findSignals(widget):
				# Don't want to catch this signal as
				# the widget is just about to die
				if signal.startswith('destroyed'):
					continue

				self.connect(
					widget, QtCore.SIGNAL(signal),
					_SignalTracker.Receiver(widget=widget, sigName=signal, callable=self._recordSignal)
				)

	def _termcode(self, num):
		return '\033[%sm'%num
	#
	# Internal Helpers
	#
	class Receiver(object):
		""" An internal helper class to handle the receiving and aggregation of all
			signals emitted from a number of widgets.

			[For an aggregator across widgets only the QSignalMapper is preferred]
		"""
		def __init__(self, widget, sigName, callable):
			super(_SignalTracker.Receiver, self).__init__()
			self._widget = widget
			self._sigName = sigName
			self._callable = callable

		def __call__(self, *args, **kwargs):
			(self._callable)(self._widget, self._sigName, args, kwargs)

	@staticmethod
	def _findSignals(widget):
		""" Find all signals defined for a given widget

			Returns a generator of signal signatures
		"""
		for idx in xrange(widget.metaObject().methodCount()):
			qtMethod = widget.metaObject().method(idx)
			if qtMethod.methodType() == QtCore.QMetaMethod.Signal:
				yield qtMethod.signature()

	def _recordSignal(self, widget, signal, args, kwargs):
		raise NotImplementedError

	def _printSignal(self, widget, signal, args, kwargs):
		self._counter += 1
		print 'Signal [%04d]: %s[%s] %s %r' % (
				self._counter,
				widget.__class__.__name__,
				widget.objectName(),
				self._termcode(91) + signal + self._termcode(0),
				args)

class SignalTracer(_SignalTracker):
	"""  A logger for Qt Signals for debugging custom widgets.

		**Example**::

			from mpc.pyqtUtils import SignalTracer
			tracer = SignalTracer()

		Create widgets and call monitor::

			comboWidget = ComboWidget()
			textEditWidget = QTextEdit()
			tracer.monitor(comboWidget, textEditWidget)

		Do something to the widgets to trigger signals::

			textEditWidget.setText('some text')
			comboWidget.setCurrentIndex(0)

		Signal details will be output as they are emitted::

			Received: QTextEdit[] textChanged(QString) ('some text',)
			Received: ComboWidget[] currentIndexChanged(QString) ('item 1',)
			Received: ComboWidget[] currentIndexChanged(int) (0,)

	"""

	def __init__(self):
		super(SignalTracer, self).__init__(showTracer=True)

	def _recordSignal(self, widget, signal, args, kwargs):
		self._printSignal( widget, signal, args, kwargs)


class SignalRecorder(_SignalTracker):
	""" A recorder for Qt Signals for debugging and for testing custom widgets.

		**Features**

			*	Handles both builtin and custom signals defined in Python
				so long as a pyqtSignal property exists.
			*	Handles overloading of signals, including builtins.
			*	Optional signalTracer (showTracer=True)

		**Example**::

			from mpc.pyqtUtils import SignalRecorder
			recorder = SignalRecorder()

		Create widgets and call monitor::

			comboWidget = ComboWidget()
			textEditWidget = QTextEdit()
			recorder.monitor(comboWidget, textEditWidget)

		Do something to the widgets to trigger signals::

			textEditWidget.setText('some text')
			comboWidget.setCurrentIndex(0)

		Verify that the signals have been emitted in correctly (in order)::

			recorder.expected('textChanged()', 'some text', widget=textEditWidget)
			recorder.expected('currentIndexChanged(QString)', 'item 1', widget=comboWidget)
			recorder.expected('currentIndexChanged(int)', 0, widget=comboWidget)

		Finally, check that no other signals were emitted::

			assert(recorder.isEmpty())
	"""

	def __init__(self, showTracer=False):
		super(SignalRecorder, self).__init__(showTracer=showTracer)
		self.__stoneTablet = None
		self.reset()

	def __len__(self):
		""" returns the number of items in the queue
		"""
		if self.__stoneTablet is not None:
			return len(self.__stoneTablet)
		return 0

	def reset(self):
		""" Clears the signal queue
		"""
		self.__stoneTablet = collections.deque()

	def playback(self):
		for item in self.__stoneTablet:
			self._printSignal(*item)

	def isEmpty(self):
		""" Returns True if the queue is empty
		"""
		return len(self.__stoneTablet or []) == 0

	def expected(self, signal, *args, **kwargs):
		""" Inspects the next recorded signal in the queue against supplied arguments
			Raises AssertionError if signal details do not match

			Args:
				signal (str) - signature of PyQt signal, eg 'currentIndexChanged(QString)'
				args         - sequence of expected values from the signal
				kwargs       - kwargs of expected signal. supply "widget=myWidget" to differentiate multiple widgets
		"""
		if self.isEmpty():
			raise AssertionError("No signals in the queue, expected %r" % signal)

		widget = kwargs.pop('widget', None)
		gotWidget, gotSignal, gotArgs, gotKwargs = self.__stoneTablet.popleft()

		if gotSignal != signal:
			raise AssertionError('Unexpected signal %r, expecting %r' % (gotSignal, signal))

		# if we are evaluating QModelIndex and QPersistentModelIndex then we cannot
		# evaluate args as tuples, the items need to be evaluated individually
		if not all (x==y for x, y in zip(args, gotArgs)):
		#if args != gotArgs:
			raise AssertionError('Unexpected args %r for signal %r, expecting %r' % (gotArgs, signal, args))

		if gotKwargs != kwargs:
			raise AssertionError('Unexpected kwargs %r for signal %r, expecting %r' % (gotKwargs, signal, kwargs))

		if widget is not None and widget != gotWidget:
			raise AssertionError('Unexpected widget %r for signal %r, expecting %r' % (gotWidget, signal, widget))

	def _recordSignal(self, widget, signal, args, kwargs):
		if self._showTracer:
			self._printSignal(widget, signal, args, kwargs)

		def _mapArgs(args):
			""" Retaining QModelIndex objects is a bad idea. Really bad. So substitute these
				objects for QPersistentModelIndex equivalents.
				The equaltity operator, in expected(), will work fine, as this is overloaded
				in QPersistentModelIndex to evaluate against QModelIndex, eg
				index == QPersistentModelIndex(index)
				>>> True
			"""
			for arg in args:
				if isinstance(arg, QtCore.QModelIndex):
					yield QtCore.QPersistentModelIndex(arg)
				else:
					yield arg

		self.__stoneTablet.append((widget, signal, list(_mapArgs(args)), kwargs))

@contextmanager
def signalRecorderContext(*args, **kwargs):
	""" Context manager for SignalRecorder

		Will capture received signals to be inspected later.
		AssertionError will be raised if any signals remain in
		the queue. All signals must be removed from queue
		using expected(), or the queue cleared using reset()

		Usage::

			from mpc.pyqtUtils.utils import signalRecorderContext
			with signalRecorderContext(widget1, widget2) as recorder:
				widget1.doSomething()
				recorder.expected('mySignal()', 3, widget=widget1)
				recorder.expected('anotherSignal()', widget=widget2)
	"""
	recorder = SignalRecorder(showTracer=kwargs.get('showTracer', False))
	recorder.monitor(*args)

	yield recorder

	if not recorder.isEmpty():
		raise AssertionError('stack is not empty: %d signals left' % len(recorder))

@contextmanager
def signalTracerContext(*args):
	""" Context manager for SignalTracer

		Will output received signals to stdout

		Usage::

			from mpc.pyqtUtils.utils import signalTracerContext
			with signalTracerContext(widget1, widget2) as tracer:
				widget1.doSomething()
	"""
	tracer = SignalTracer()
	tracer.monitor(*args)
	yield tracer



from contextlib  import contextmanager
from version import __VERSION__
from PyQt4 import QtGui, QtCore, QtSql
from utils import showWaitCursorDecorator, showWaitCursor
from models import RecordModel, SortProxyModel, CheckComboModel
from csvdecoder import Decoder, DecoderException
from database import db
from ui_pydosh import Ui_pydosh
from dialogs import SettingsDialog, TagDialog, ImportDialog
import enum
import pydosh_rc


class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
	def __init__(self, parent=None):
		super(PydoshWindow, self).__init__(parent=parent)
		self.setupUi(self)
		self.model = None

		self.setAttribute(QtCore.Qt.WA_DeleteOnClose)

		self.checkedCombo.addItem('all', enum.kCheckedStatus_All)
		self.checkedCombo.addItem('checked', enum.kCheckedStatus_Checked)
		self.checkedCombo.addItem('unchecked', enum.kCheckedStatus_UnChecked)

		self.inoutCombo.addItem('all', enum.kInOutStatus_All)
		self.inoutCombo.addItem('in', enum.kInOutStatus_In)
		self.inoutCombo.addItem('out', enum.kInOutStatus_Out)

		self.dateCombo.addItem('All', userData=enum.kDate_All)
		self.dateCombo.addItem('Last 12 months', userData=enum.kDate_PreviousYear)
		self.dateCombo.addItem('Last month', userData=enum.kDate_PreviousMonth)
		self.dateCombo.addItem('Last import', userData=enum.kdate_LastImport)

		self.toggleCheckButton.setEnabled(False)
		self.deleteButton.setEnabled(False)

		self.accountCombo.setDefaultText('all')
		self.tagCombo.selectionChanged.connect(self.setFilter)
		self.checkedCombo.currentIndexChanged.connect(self.setFilter)
		self.inoutCombo.currentIndexChanged.connect(self.setFilter)
		self.accountCombo.selectionChanged.connect(self.setFilter)
		self.tagCombo.selectionChanged.connect(self.setFilter)
		self.descEdit.textChanged.connect(self.setFilter)
		self.amountEdit.textChanged.connect(self.setFilter)
		self.amountEdit.controlKeyPressed.connect(self.controlKeyPressed)
		self.startDateEdit.dateChanged.connect(self.setFilter)
		self.endDateEdit.dateChanged.connect(self.setFilter)
		self.toggleCheckButton.clicked.connect(self.toggleSelected)
		self.deleteButton.clicked.connect(self.deleteRecords)
		self.dateCombo.currentIndexChanged.connect(self.setDate)
		self.startDateEdit.dateChanged.connect(self.setDate)
		self.endDateEdit.dateChanged.connect(self.setDate)
		self.reloadButton.clicked.connect(self.reset)
		self.tagEditButton.pressed.connect(self.addTagButtonPressed)

		db.connected.connect(self.setConnectionStatus)

		amountValidator = QtGui.QRegExpValidator(QtCore.QRegExp("[<>=0-9.]*"), self)
		self.amountEdit.setValidator(amountValidator)

		self.startDateEdit.setCalendarPopup(True)
		self.endDateEdit.setCalendarPopup(True)

		self.addActions()

		self.inTotalLabel.setFrameStyle(QtGui.QFrame.StyledPanel | QtGui.QFrame.Sunken)
		self.outTotalLabel.setFrameStyle(QtGui.QFrame.StyledPanel | QtGui.QFrame.Sunken)
		self.recordCountLabel.setFrameStyle(QtGui.QFrame.StyledPanel | QtGui.QFrame.Sunken)

		self.setConnectionStatus(db.isConnected)

		self.__signalsToBlock = (
				self.accountCombo,
				self.checkedCombo,
				self.inoutCombo,
				self.tagCombo,
				self.dateCombo,
				self.descEdit,
				self.amountEdit,
				self.startDateEdit,
				self.endDateEdit,
		)

		with self.blockAllSignals():

			self.model = None
			recordsModel = RecordModel(db.userId, self)
			recordsModel.setTable("records")
			recordsModel.setEditStrategy(QtSql.QSqlTableModel.OnManualSubmit)
			recordsModel.select()
			self.model = recordsModel

			proxyModel = SortProxyModel(self)
			proxyModel.setSourceModel(recordsModel)
			proxyModel.setFilterKeyColumn(-1)

			self.tableView.setEditTriggers(QtGui.QAbstractItemView.DoubleClicked | QtGui.QAbstractItemView.SelectedClicked)
			self.tableView.setModel(proxyModel)
			self.tableView.verticalHeader().hide()
			self.tableView.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
			self.tableView.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
			self.tableView.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)

			self.tableView.setColumnHidden(enum.kRecordColumn_RecordId, True)
			self.tableView.setColumnHidden(enum.kRecordColumn_CheckDate, True)
			self.tableView.setColumnHidden(enum.kRecordColumn_RawData, True)
			self.tableView.setColumnHidden(enum.kRecordColumn_InsertDate, True)
			self.tableView.setSortingEnabled(True)
			self.tableView.sortByColumn(enum.kRecordColumn_Date, QtCore.Qt.DescendingOrder)
			self.tableView.horizontalHeader().setStretchLastSection(True)
			self.tableView.selectionModel().selectionChanged.connect(self.activateButtons)

			# Set sql data model for account types
			self.accountCombo.clear()
	
			accountModel = CheckComboModel()
			accountModel.setTable('accounttypes')
			accountModel.setFilter("""
				accounttypeid IN (
					SELECT distinct accounttypeid
					FROM records
					WHERE userid=%d)
				""" % db.userId)

			accountModel.select()
			accountModel.setUserRoleColumn(enum.kAccountTypeColumn_AccountTypeId)
			self.accountCombo.setModelColumn(enum.kAccountTypeColumn_AccountName)
			self.accountCombo.setModel(accountModel)

			# Set tag model
			tagModel = CheckComboModel()
			tagModel.setTable('tags')
	
			# Only display tags that have been set on the current records
			tagModel.setFilter("""
				userid=%d
				AND tagid IN (
					SELECT DISTINCT tagid
					FROM recordtags rt
					INNER JOIN records r
					ON r.recordid=rt.recordid
					)
				""" % db.userId)
			tagModel.select()
			tagModel.setUserRoleColumn(enum.kTagsColumn_TagId)
			self.tagCombo.setModelColumn(enum.kTagsColumn_TagName)
			self.tagCombo.setModel(tagModel)

			self.reset()

	def showAbout(self):

		QtGui.QMessageBox.about(self,
			'About pydosh',
			"<html><p><h2>pydosh</h2></p>"
			"<p>version %s</p>"
			"<p>by Will Hall <a href=\"mailto:will@innerhippy.com\">will@innerhippy.com</a></p>"
			"<p>Copywrite (c) 2013.</p>"
			"<p>Written using PyQt %s</p>"
			"<p><a href=\"http://www.innerhippy.com\">www.innerhippy.com</a></p>"
			"enjoy!</html>" % (__VERSION__, QtCore.QT_VERSION_STR)
			)

	def setConnectionStatus(self, isConnected):
		if isConnected:
			self.connectionStatusText.setText('connected to %s@%s' % (db.database, db.hostname))

		else:
			self.connectionStatusText.setText('not connected')
			self.model = None

			self.tableView.setModel(None)
			self.displayRecordCount()

	def setDate(self):
		selected = self.dateCombo.itemData(self.dateCombo.currentIndex(), QtCore.Qt.UserRole).toPyObject()

		if selected == enum.kDate_All:
			self.startDateEdit.setDate(self.startDateEdit.minimumDate())
			self.endDateEdit.setDate(self.endDateEdit.maximumDate())
			self.startDateEdit.setEnabled(True)
			self.endDateEdit.setEnabled(True)
		elif selected == enum.kDate_PreviousMonth:
			self.startDateEdit.setDate(self.endDateEdit.date().addMonths(-1))
			self.endDateEdit.setEnabled(True)
			self.startDateEdit.setEnabled(False)
		elif selected == enum.kDate_PreviousYear:
			self.startDateEdit.setDate(self.endDateEdit.date().addYears(-1))
			self.startDateEdit.setEnabled(True)
			self.endDateEdit.setEnabled(True)
		elif selected == enum.kdate_LastImport:
			self.startDateEdit.setEnabled(False)
			self.endDateEdit.setEnabled(False)

		self.setFilter()

	def settingsDialog(self):
		dialog = SettingsDialog(self)
		dialog.exec_()
		self.setFilter()

	def addTagButtonPressed(self):
		# Get recordids from all selected rows
		selectionModel = self.tableView.selectionModel()
		if selectionModel is None:
			return []

		proxyModel = self.tableView.model()
		recordIds = []
		for proxyIndex in selectionModel.selectedRows():
			index = self.model.index(proxyModel.mapToSource(proxyIndex).row(), enum.kRecordColumn_RecordId) 
			recordIds.append(index.data().toPyObject())

		dialog = TagDialog(recordIds, self)
		if dialog.exec_():
			self.tagCombo.model().select()

	def importDialog(self):
		if not db.isConnected:
			return

		combo = QtGui.QComboBox(self)
		model = QtSql.QSqlTableModel(self)
		model.setTable('accounttypes')
		model.select()
		combo.setModel(model)
		combo.setModelColumn(enum.kAccountTypeColumn_AccountName)

		# find the previous accounttype that was used (if any)
		settings = QtCore.QSettings()
		accounttype = settings.value("options/importaccounttype").toString()

		index = combo.findText(accounttype)

		if index != -1:
			combo.setCurrentIndex(index)
		else:
			combo.setCurrentIndex(0)

		label = QtGui.QLabel('Account type:')
		importDir = settings.value("options/importdirectory").toString()

		if importDir.isEmpty():
			importDir = QtCore.QDir.homePath()

		dialog = QtGui.QFileDialog(self, 'Open File', importDir, "*.csv")
		dialog.setFileMode(QtGui.QFileDialog.ExistingFiles)
		dialog.setOption(QtGui.QFileDialog.DontUseNativeDialog)

		gridbox = dialog.layout()

		if gridbox:
			gridbox.addWidget(label)
			gridbox.addWidget(combo)

		if not dialog.exec_():
			return

		accountId, ok = combo.model().index(combo.currentIndex(), enum.kAccountTypeColumn_AccountTypeId).data().toInt()
		if not ok:
			QtGui.QMessageBox.critical(self, 'Import Error', 'No account type specified', QtGui.QMessageBox.Ok)
			return

		fileNames = QtCore.QStringList([QtCore.QFileInfo(f).fileName() for f in dialog.selectedFiles()]) 
		dateField = combo.model().index(combo.currentIndex(), enum.kAccountTypeColumn_DateField).data()
		descriptionField = combo.model().index(combo.currentIndex(), enum.kAccountTypeColumn_DescriptionField).data()
		creditField = combo.model().index(combo.currentIndex(), enum.kAccountTypeColumn_CreditField).data()
		debitField = combo.model().index(combo.currentIndex(), enum.kAccountTypeColumn_DebitField).data()
		currencySign = combo.model().index(combo.currentIndex(), enum.kAccountTypeColumn_CurrencySign).data()
		dateFormat = combo.model().index(combo.currentIndex(), enum.kAccountTypeColumn_DateFormat).data()

		# Save the settings for next time
		settings.setValue('options/importaccounttype', combo.currentText())
		settings.setValue('options/importdirectory', dialog.directory().absolutePath())

		with showWaitCursor():
			try:
				decoder = Decoder(
						dateField,
						descriptionField,
						creditField,
						debitField,
						currencySign,
						dateFormat,
						dialog.selectedFiles())
			except DecoderException, exc:
				QtGui.QMessageBox.critical(self, 'Import Error', str(exc), QtGui.QMessageBox.Ok)
				return

			dialog = ImportDialog(decoder.records, accountId, self)
			dialog.setWindowTitle(fileNames.join(', '))

		if dialog.exec_():
			self.accountCombo.model().select()
			self.reset()

	def activateButtons(self):

		model = self.tableView.selectionModel()
		enable = False

		if model:
			enable = len(model.selectedRows()) > 0

		self.toggleCheckButton.setEnabled(enable)
		self.deleteButton.setEnabled(enable)


	def controlKeyPressed(self, key):
		""" Control key has been pressed
		
			If we have a single row displayed, then toggle the status
			If this results in the model having no records then
			reset the QLineEdit that triggered the call.
		"""
		if self.model and key == QtCore.Qt.Key_Space:
			if self.model.rowCount() == 1:
				self.tableView.selectAll()
				self.toggleSelected()

			if self.model.rowCount() == 0 and isinstance(self.sender(), QtGui.QLineEdit):
					self.sender().clear()

	def deleteRecords(self):
		""" Delete selected records
		"""
		selectionModel = self.tableView.selectionModel()
		proxyModel = self.tableView.model()
		dataModel = proxyModel.sourceModel()

		if QtGui.QMessageBox.question(
				self, 'Delete Records',
				'Are you sure you want to delete %d rows?' % len(selectionModel.selectedRows()), 
				QtGui.QMessageBox.Yes|QtGui.QMessageBox.No) != QtGui.QMessageBox.Yes:
			return

		with showWaitCursor():
			for index in selectionModel.selectedRows():
				dataModel.removeRow(proxyModel.mapToSource(index).row())
	
			dataModel.submitAll()
			self.accountCombo.model().select()
			self.tagCombo.model().select()

	@contextmanager
	def blockAllSignals(self):
		try:
			for widget in self.__signalsToBlock:
				widget.blockSignals(True)
			yield
		finally:
			for widget in self.__signalsToBlock:
				widget.blockSignals(False)


	def populateDates(self):
		query = QtSql.QSqlQuery("""
				SELECT MIN(date), MAX(date)
				FROM records
				WHERE userid=%d
			""" % db.userId)

		if query.next():
			startDate = query.value(0).toDate()
			endDate = query.value(1).toDate()

			self.startDateEdit.setDateRange(startDate, endDate)
			self.endDateEdit.setDateRange(startDate, endDate)

			self.startDateEdit.setDate(startDate)
			self.endDateEdit.setDate(endDate)

	@showWaitCursorDecorator
	def toggleSelected(self, *args):

		selectionModel = self.tableView.selectionModel()

		if selectionModel is None:
			return

		proxyModel = self.tableView.model()

		for proxyIndex in reversed(selectionModel.selectedRows()):
			index = self.model.index(proxyModel.mapToSource(proxyIndex).row(), enum.kRecordColumn_Checked) 
			checkState = index.data(QtCore.Qt.CheckStateRole).toPyObject()
			newState = QtCore.Qt.Unchecked if checkState == QtCore.Qt.Checked else QtCore.Qt.Checked
			self.model.setData(index, QtCore.QVariant(newState), QtCore.Qt.CheckStateRole)

		self.displayRecordCount()

	def reset(self):
		if self.model is None or not db.isConnected:
			return

		with self.blockAllSignals():
			self.populateDates()
			self.checkedCombo.setCurrentIndex(enum.kCheckedStatus_All)
			self.dateCombo.setCurrentIndex(enum.kDate_PreviousYear)
			self.setDate()
			self.tagCombo.clearAll()
			self.accountCombo.clearAll()
			self.inoutCombo.setCurrentIndex(enum.kInOutStatus_All)
			self.amountEdit.clear()
			self.descEdit.clear()
			self.endDateEdit.setEnabled(True)
			self.tableView.sortByColumn(enum.kRecordColumn_Date, QtCore.Qt.DescendingOrder)

		self.setFilter()

	def addActions(self):
		quitAction = QtGui.QAction(QtGui.QIcon(':/icons/exit.png'), '&Quit', self)
		quitAction.setShortcut('Alt+q')
		quitAction.setStatusTip('Exit the program')
		quitAction.triggered.connect(self.close)

		settingsAction = QtGui.QAction(QtGui.QIcon(':/icons/wrench.png'), '&Settings', self)
		settingsAction.setShortcut('Alt+s')
		settingsAction.setStatusTip('Change the settings')
		settingsAction.triggered.connect(self.settingsDialog)

		importAction = QtGui.QAction(QtGui.QIcon(':/icons/import.png'), '&Import', self)
		importAction.setShortcut('Alt+i')
		importAction.setStatusTip('Import Bank statements')
		importAction.triggered.connect(self.importDialog)

		aboutAction = QtGui.QAction(QtGui.QIcon(':/icons/help.png'), '&About', self)
		aboutAction.setStatusTip('About')
		aboutAction.triggered.connect(self.showAbout)

		self.addAction(settingsAction)
		self.addAction(importAction)
		self.addAction(quitAction)
		self.addAction(aboutAction)

		# File menu
		fileMenu = self.menuBar().addMenu('&Tools')
		fileMenu.addAction(settingsAction)
		fileMenu.addAction(importAction)
		fileMenu.addAction(quitAction)

		helpMenu = self.menuBar().addMenu('&Help')
		helpMenu.addAction(aboutAction)


	def displayRecordCount(self):
		numRecords = 0
		totalRecords = 0
		inTotal = 0.0
		outTotal = 0.0

		if self.model is not None:
			numRecords = self.model.rowCount()

			query = QtSql.QSqlQuery('SELECT COUNT(*) FROM records WHERE userid=%d' % db.userId)
			query.next()
			totalRecords = query.value(0).toPyObject()

			for i in xrange(numRecords):
				amount, _ = self.model.record(i).value(enum.kRecordColumn_Amount).toDouble()
				if amount > 0.0:
					inTotal += amount
				else:
					outTotal += abs(amount)

		self.inTotalLabel.setText(QtCore.QString("%L1").arg(inTotal, 0, 'f', 2))
		self.outTotalLabel.setText(QtCore.QString("%L1").arg(outTotal, 0, 'f', 2))
		self.recordCountLabel.setText('%d / %d' % (numRecords, totalRecords))

	@showWaitCursorDecorator
	def setFilter(self, *args):
		if self.model is None or not db.isConnected:
			return

		queryFilter = []

		# Account filter
		accountIds = [index.data(QtCore.Qt.UserRole).toPyObject() for index in self.accountCombo.checkedIndexes()]
		if accountIds:
			queryFilter.append('r.accounttypeid in (%s)' % ', '.join(str(acid) for acid in accountIds))

		# Date filter
		if self.dateCombo.currentIndex() == enum.kdate_LastImport:
			queryFilter.append("""
				r.insertdate = (
					SELECT MAX(insertdate) 
					FROM records)
			""")
		else:
			startDate = self.startDateEdit.date()
			endDate = self.endDateEdit.date()
	
			if startDate.isValid() and endDate.isValid():
				queryFilter.append("r.date >= '%s'" % startDate.toString(QtCore.Qt.ISODate))
				queryFilter.append("r.date <= '%s'" % endDate.toString(QtCore.Qt.ISODate))

		# checked state filter
		state = self.checkedCombo.itemData(self.checkedCombo.currentIndex(), QtCore.Qt.UserRole).toPyObject()
		if state == enum.kCheckedStatus_Checked:
			queryFilter.append('r.checked=1')
		elif state == enum.kCheckedStatus_UnChecked:
			queryFilter.append('r.checked=0')

		# money in/out filter
		state = self.inoutCombo.itemData(self.inoutCombo.currentIndex(), QtCore.Qt.UserRole).toPyObject()
		if state == enum.kInOutStatus_In:
			queryFilter.append('r.amount > 0')
		elif state == enum.kInOutStatus_Out:
			queryFilter.append('r.amount < 0')

		# description filter
		if self.descEdit.text():
			queryFilter.append("lower(r.description) LIKE '%%%s%%'" % self.descEdit.text().toLower())

		# amount filter. May contain operators < > <= or >=
		amountFilter = self.amountEdit.text()
		if amountFilter:
			if amountFilter.contains(QtCore.QRegExp('[<>=]+')):
				# looks like we have operators, test validity and amount
				rx = QtCore.QRegExp('^(=|>|<|>=|<=)([\\.\\d+]+)')
				if rx.indexIn(amountFilter) != -1:
					queryFilter.append('abs(r.amount) %s %s' % (rx.cap(1), rx.cap(2)))
				else:
					# Input not complete yet.
					return
			else:
				# No operator supplied - treat amount as a string
				queryFilter.append(
					"(CAST(r.amount AS char(10)) LIKE '%s%%' OR CAST(r.amount AS char(10)) LIKE '-%s%%')" %
					(amountFilter, amountFilter))

		# tag filter
		tagIds = [index.data(QtCore.Qt.UserRole).toPyObject() for index in self.tagCombo.checkedIndexes()]

		if tagIds:
			queryFilter.append("""
				r.recordid IN (
					SELECT recordid
					FROM recordtags
					WHERE tagid in (%s))
				""" % ', '.join([str(tagid) for tagid in tagIds]))

		self.model.setFilter('\nAND '.join(queryFilter))
#		print self.model.query().lastQuery().replace(' AND ', '').replace('\n', ' ')

		self.tableView.resizeColumnsToContents()
		self.displayRecordCount()




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/pydosh.ui'
#
# Created: Sat Mar  2 16:27:20 2013
#      by: PyQt4 UI code generator 4.9.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_pydosh(object):
    def setupUi(self, pydosh):
        pydosh.setObjectName(_fromUtf8("pydosh"))
        pydosh.resize(838, 707)
        self.centralwidget = QtGui.QWidget(pydosh)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.gridLayout_4 = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout_4.setObjectName(_fromUtf8("gridLayout_4"))
        self.widget_3 = QtGui.QWidget(self.centralwidget)
        self.widget_3.setObjectName(_fromUtf8("widget_3"))
        self.gridLayout_3 = QtGui.QGridLayout(self.widget_3)
        self.gridLayout_3.setMargin(0)
        self.gridLayout_3.setObjectName(_fromUtf8("gridLayout_3"))
        self.groupBox = QtGui.QGroupBox(self.widget_3)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.gridLayout = QtGui.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label_3 = QtGui.QLabel(self.groupBox)
        self.label_3.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_3.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.checkedCombo = QtGui.QComboBox(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkedCombo.sizePolicy().hasHeightForWidth())
        self.checkedCombo.setSizePolicy(sizePolicy)
        self.checkedCombo.setObjectName(_fromUtf8("checkedCombo"))
        self.gridLayout.addWidget(self.checkedCombo, 0, 1, 1, 1)
        self.label_4 = QtGui.QLabel(self.groupBox)
        self.label_4.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.gridLayout.addWidget(self.label_4, 2, 0, 1, 1)
        self.accountCombo = MultiComboBox(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountCombo.sizePolicy().hasHeightForWidth())
        self.accountCombo.setSizePolicy(sizePolicy)
        self.accountCombo.setObjectName(_fromUtf8("accountCombo"))
        self.gridLayout.addWidget(self.accountCombo, 2, 1, 1, 1)
        self.inoutCombo = QtGui.QComboBox(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.inoutCombo.sizePolicy().hasHeightForWidth())
        self.inoutCombo.setSizePolicy(sizePolicy)
        self.inoutCombo.setObjectName(_fromUtf8("inoutCombo"))
        self.gridLayout.addWidget(self.inoutCombo, 1, 1, 1, 1)
        self.label_13 = QtGui.QLabel(self.groupBox)
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.gridLayout.addWidget(self.label_13, 1, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox, 0, 1, 1, 1)
        self.groupBox_3 = QtGui.QGroupBox(self.widget_3)
        self.groupBox_3.setMinimumSize(QtCore.QSize(320, 0))
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self.gridLayout_2 = QtGui.QGridLayout(self.groupBox_3)
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        self.label_14 = QtGui.QLabel(self.groupBox_3)
        self.label_14.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_14.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.gridLayout_2.addWidget(self.label_14, 0, 0, 1, 1)
        self.descEdit = SearchLineEdit(self.groupBox_3)
        self.descEdit.setObjectName(_fromUtf8("descEdit"))
        self.gridLayout_2.addWidget(self.descEdit, 0, 1, 1, 1)
        self.label_6 = QtGui.QLabel(self.groupBox_3)
        self.label_6.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.gridLayout_2.addWidget(self.label_6, 1, 0, 1, 1)
        self.amountEdit = SearchLineEdit(self.groupBox_3)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.amountEdit.sizePolicy().hasHeightForWidth())
        self.amountEdit.setSizePolicy(sizePolicy)
        self.amountEdit.setObjectName(_fromUtf8("amountEdit"))
        self.gridLayout_2.addWidget(self.amountEdit, 1, 1, 1, 1)
        self.label_9 = QtGui.QLabel(self.groupBox_3)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.gridLayout_2.addWidget(self.label_9, 2, 0, 1, 1)
        self.tagCombo = MultiComboBox(self.groupBox_3)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tagCombo.sizePolicy().hasHeightForWidth())
        self.tagCombo.setSizePolicy(sizePolicy)
        self.tagCombo.setObjectName(_fromUtf8("tagCombo"))
        self.gridLayout_2.addWidget(self.tagCombo, 2, 1, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_3, 0, 2, 1, 1)
        spacerItem = QtGui.QSpacerItem(126, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 0, 3, 1, 1)
        self.groupBox_2 = QtGui.QGroupBox(self.widget_3)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.gridLayout_5 = QtGui.QGridLayout(self.groupBox_2)
        self.gridLayout_5.setObjectName(_fromUtf8("gridLayout_5"))
        self.label = QtGui.QLabel(self.groupBox_2)
        self.label.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout_5.addWidget(self.label, 0, 0, 1, 1)
        self.endDateEdit = QtGui.QDateEdit(self.groupBox_2)
        self.endDateEdit.setObjectName(_fromUtf8("endDateEdit"))
        self.gridLayout_5.addWidget(self.endDateEdit, 0, 1, 1, 2)
        self.label_2 = QtGui.QLabel(self.groupBox_2)
        self.label_2.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout_5.addWidget(self.label_2, 1, 0, 1, 1)
        self.startDateEdit = QtGui.QDateEdit(self.groupBox_2)
        self.startDateEdit.setObjectName(_fromUtf8("startDateEdit"))
        self.gridLayout_5.addWidget(self.startDateEdit, 1, 1, 1, 2)
        self.dateCombo = QtGui.QComboBox(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dateCombo.sizePolicy().hasHeightForWidth())
        self.dateCombo.setSizePolicy(sizePolicy)
        self.dateCombo.setObjectName(_fromUtf8("dateCombo"))
        self.gridLayout_5.addWidget(self.dateCombo, 2, 1, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_2, 0, 0, 1, 1)
        self.gridLayout_4.addWidget(self.widget_3, 0, 0, 1, 1)
        self.widget = QtGui.QWidget(self.centralwidget)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout_2.setMargin(0)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.toggleCheckButton = QtGui.QToolButton(self.widget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/accept.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toggleCheckButton.setIcon(icon)
        self.toggleCheckButton.setObjectName(_fromUtf8("toggleCheckButton"))
        self.horizontalLayout_2.addWidget(self.toggleCheckButton)
        self.tagEditButton = QtGui.QToolButton(self.widget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/tag_yellow.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tagEditButton.setIcon(icon1)
        self.tagEditButton.setObjectName(_fromUtf8("tagEditButton"))
        self.horizontalLayout_2.addWidget(self.tagEditButton)
        self.reloadButton = QtGui.QToolButton(self.widget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/arrow_refresh.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.reloadButton.setIcon(icon2)
        self.reloadButton.setObjectName(_fromUtf8("reloadButton"))
        self.horizontalLayout_2.addWidget(self.reloadButton)
        self.deleteButton = QtGui.QToolButton(self.widget)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/cross.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.deleteButton.setIcon(icon3)
        self.deleteButton.setObjectName(_fromUtf8("deleteButton"))
        self.horizontalLayout_2.addWidget(self.deleteButton)
        spacerItem1 = QtGui.QSpacerItem(703, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.gridLayout_4.addWidget(self.widget, 1, 0, 1, 1)
        self.tableView = QtGui.QTableView(self.centralwidget)
        self.tableView.setObjectName(_fromUtf8("tableView"))
        self.gridLayout_4.addWidget(self.tableView, 2, 0, 1, 1)
        self.widget_2 = QtGui.QWidget(self.centralwidget)
        self.widget_2.setObjectName(_fromUtf8("widget_2"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.widget_2)
        self.horizontalLayout_3.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.connectionStatusText = QtGui.QLabel(self.widget_2)
        self.connectionStatusText.setObjectName(_fromUtf8("connectionStatusText"))
        self.horizontalLayout_3.addWidget(self.connectionStatusText)
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.label_10 = QtGui.QLabel(self.widget_2)
        self.label_10.setObjectName(_fromUtf8("label_10"))
        self.horizontalLayout_3.addWidget(self.label_10)
        self.inTotalLabel = QtGui.QLabel(self.widget_2)
        self.inTotalLabel.setObjectName(_fromUtf8("inTotalLabel"))
        self.horizontalLayout_3.addWidget(self.inTotalLabel)
        spacerItem3 = QtGui.QSpacerItem(28, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.label_11 = QtGui.QLabel(self.widget_2)
        self.label_11.setObjectName(_fromUtf8("label_11"))
        self.horizontalLayout_3.addWidget(self.label_11)
        self.outTotalLabel = QtGui.QLabel(self.widget_2)
        self.outTotalLabel.setObjectName(_fromUtf8("outTotalLabel"))
        self.horizontalLayout_3.addWidget(self.outTotalLabel)
        spacerItem4 = QtGui.QSpacerItem(134, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.label_12 = QtGui.QLabel(self.widget_2)
        self.label_12.setObjectName(_fromUtf8("label_12"))
        self.horizontalLayout_3.addWidget(self.label_12)
        self.recordCountLabel = QtGui.QLabel(self.widget_2)
        self.recordCountLabel.setObjectName(_fromUtf8("recordCountLabel"))
        self.horizontalLayout_3.addWidget(self.recordCountLabel)
        self.gridLayout_4.addWidget(self.widget_2, 3, 0, 1, 1)
        pydosh.setCentralWidget(self.centralwidget)
        self.label_14.setBuddy(self.descEdit)
        self.label_6.setBuddy(self.amountEdit)

        self.retranslateUi(pydosh)
        QtCore.QMetaObject.connectSlotsByName(pydosh)

    def retranslateUi(self, pydosh):
        pydosh.setWindowTitle(QtGui.QApplication.translate("pydosh", "doshLogger", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("pydosh", "filter", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("pydosh", "checked", None, QtGui.QApplication.UnicodeUTF8))
        self.checkedCombo.setToolTip(QtGui.QApplication.translate("pydosh", "filter on checked status", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("pydosh", "account", None, QtGui.QApplication.UnicodeUTF8))
        self.accountCombo.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>filter on account type</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.inoutCombo.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>filter on money in or out</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_13.setText(QtGui.QApplication.translate("pydosh", "in/out", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("pydosh", "search", None, QtGui.QApplication.UnicodeUTF8))
        self.label_14.setText(QtGui.QApplication.translate("pydosh", "&description", None, QtGui.QApplication.UnicodeUTF8))
        self.descEdit.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>filter on description (Alt-d)</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("pydosh", "&amount", None, QtGui.QApplication.UnicodeUTF8))
        self.amountEdit.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>filter by amount (Alt-a)</p><p>Accepts operators = &gt; &gt;= &lt; &lt;=</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("pydosh", "tag", None, QtGui.QApplication.UnicodeUTF8))
        self.tagCombo.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>filter on tags</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("pydosh", "date", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("pydosh", "end", None, QtGui.QApplication.UnicodeUTF8))
        self.endDateEdit.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>Set the end date filter</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("pydosh", "start", None, QtGui.QApplication.UnicodeUTF8))
        self.startDateEdit.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>Set the start date filter</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.dateCombo.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>Select a date filter</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.toggleCheckButton.setToolTip(QtGui.QApplication.translate("pydosh", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Arial\'; font-size:10pt;\">toggle checked</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.toggleCheckButton.setText(QtGui.QApplication.translate("pydosh", "&t", None, QtGui.QApplication.UnicodeUTF8))
        self.tagEditButton.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>edit tags</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.tagEditButton.setText(QtGui.QApplication.translate("pydosh", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.reloadButton.setToolTip(QtGui.QApplication.translate("pydosh", "reload data (Alt-r)", None, QtGui.QApplication.UnicodeUTF8))
        self.reloadButton.setText(QtGui.QApplication.translate("pydosh", "&r", None, QtGui.QApplication.UnicodeUTF8))
        self.deleteButton.setToolTip(QtGui.QApplication.translate("pydosh", "<html><head/><body><p>delete record(s)</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.deleteButton.setText(QtGui.QApplication.translate("pydosh", "&r", None, QtGui.QApplication.UnicodeUTF8))
        self.connectionStatusText.setText(QtGui.QApplication.translate("pydosh", "TextLabel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_10.setText(QtGui.QApplication.translate("pydosh", "in:", None, QtGui.QApplication.UnicodeUTF8))
        self.inTotalLabel.setToolTip(QtGui.QApplication.translate("pydosh", "total incoming for filtered records", None, QtGui.QApplication.UnicodeUTF8))
        self.inTotalLabel.setText(QtGui.QApplication.translate("pydosh", "TextLabel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_11.setText(QtGui.QApplication.translate("pydosh", "out:", None, QtGui.QApplication.UnicodeUTF8))
        self.outTotalLabel.setToolTip(QtGui.QApplication.translate("pydosh", "total outgoing for filtered records", None, QtGui.QApplication.UnicodeUTF8))
        self.outTotalLabel.setText(QtGui.QApplication.translate("pydosh", "TextLabel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_12.setText(QtGui.QApplication.translate("pydosh", "showing:", None, QtGui.QApplication.UnicodeUTF8))
        self.recordCountLabel.setToolTip(QtGui.QApplication.translate("pydosh", "filtered records / total records", None, QtGui.QApplication.UnicodeUTF8))
        self.recordCountLabel.setText(QtGui.QApplication.translate("pydosh", "TextLabel", None, QtGui.QApplication.UnicodeUTF8))

from searchlineedit import SearchLineEdit
from multicombobox import MultiComboBox
import pydosh_rc



from contextlib  import contextmanager
from PyQt4 import QtGui, QtCore

def showWaitCursorDecorator(f):
	""" Decorator for display a wait cursor whilst in a slow function
	"""
	def new_f(*args, **kwargs):
		QtGui.QApplication.setOverrideCursor( QtCore.Qt.WaitCursor )
		try:
			return f(*args, **kwargs)
		finally:
			QtGui.QApplication.restoreOverrideCursor()
	new_f.__name__ = f.__name__
	new_f.__doc__  = f.__doc__
	return new_f

	def blockAllSignals(self):
		try:
			for widget in self.__signalsToBlock:
				widget.blockSignals(True)
			yield
		finally:
			for widget in self.__signalsToBlock:
				widget.blockSignals(False)

@contextmanager
def showWaitCursor():
	""" Context manager utility for showing wait cursor for a code block
	"""
	try:
		QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
		yield
	finally:
		QtGui.QApplication.restoreOverrideCursor()



from setuptools import setup
from pydosh import version

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='pydosh',
	version=version.__VERSION__,
	description='Bank statement transaction manager, written in PyQt',
	long_description=readme(),
	url='http://github.com/innerhippy/pydosh',
	author='Will Hall',
	author_email='will@innerhippy.com',
	license='LICENSE.txt',
	packages=['pydosh'],
    data_files=[
        ('share/applications', ['pydosh.desktop']),
        ('share/pixmaps', ['ui/icons/pydosh.png', 'ui/icons/pydosh.xpm']),
    ],
    scripts = ['scripts/pydosh'],
	zip_safe=False)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/tags.ui'
#
# Created: Sat Mar  2 16:27:20 2013
#      by: PyQt4 UI code generator 4.9.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_Tags(object):
    def setupUi(self, Tags):
        Tags.setObjectName(_fromUtf8("Tags"))
        Tags.resize(223, 267)
        self.gridLayout_2 = QtGui.QGridLayout(Tags)
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        self.widget = QtGui.QWidget(Tags)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.gridLayout = QtGui.QGridLayout(self.widget)
        self.gridLayout.setMargin(0)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.addTagButton = QtGui.QToolButton(self.widget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/add.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.addTagButton.setIcon(icon)
        self.addTagButton.setObjectName(_fromUtf8("addTagButton"))
        self.gridLayout.addWidget(self.addTagButton, 0, 1, 1, 1)
        self.deleteTagButton = QtGui.QToolButton(self.widget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/delete.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.deleteTagButton.setIcon(icon1)
        self.deleteTagButton.setObjectName(_fromUtf8("deleteTagButton"))
        self.gridLayout.addWidget(self.deleteTagButton, 1, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 173, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 1, 1, 1)
        self.tagView = QtGui.QListView(self.widget)
        self.tagView.setObjectName(_fromUtf8("tagView"))
        self.gridLayout.addWidget(self.tagView, 0, 0, 3, 1)
        self.gridLayout_2.addWidget(self.widget, 0, 0, 1, 2)
        self.buttonBox = QtGui.QDialogButtonBox(Tags)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Save)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.gridLayout_2.addWidget(self.buttonBox, 1, 1, 1, 1)

        self.retranslateUi(Tags)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), Tags.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), Tags.reject)
        QtCore.QMetaObject.connectSlotsByName(Tags)

    def retranslateUi(self, Tags):
        Tags.setWindowTitle(QtGui.QApplication.translate("Tags", "Tag Editor", None, QtGui.QApplication.UnicodeUTF8))
        self.addTagButton.setText(QtGui.QApplication.translate("Tags", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.deleteTagButton.setText(QtGui.QApplication.translate("Tags", "...", None, QtGui.QApplication.UnicodeUTF8))

import pydosh_rc



from copy import deepcopy
from PyQt4 import QtCore, QtGui, QtSql
import enum
from database import db
import pydosh_rc

class ImportException(Exception):
	""" General exception for record import
	"""

class ImportRecord(object):
	def __init__(self, *args):
		super(ImportRecord, self).__init__()
		self.data, self.date, self.desc, self.txdate, self.debit, self.credit, self.error = args
		self.__isImported = False

	@property
	def valid(self):
		return self.error is None

	@property
	def imported(self):
		return self.__isImported

	@imported.setter
	def imported(self, value):
		self.__isImported = value

	@property
	def checksum(self):
		return QtCore.QString(QtCore.QCryptographicHash.hash(self.data, QtCore.QCryptographicHash.Md5).toHex())

	def __eq__(self, other):
		return self.checksum == other.checksum

	def __str__(self):
		return 'date=%r txdate=%r debit=%r credit=%r desc=%r' % (self.date, self.txdate, self.credit, self.debit, self.desc)

	def __repr__(self):
		return '%r' % str(self)

class ImportModel(QtCore.QAbstractTableModel):
	def __init__(self, parent=None):
		super(ImportModel, self).__init__(parent=parent)
		self.__records = []
		self.__recordsRollback = None
		self.dataSaved = False
		self.__currentTimestamp = None

	def saveRecord(self, accountId, index):
		""" Saves the import record to the database
			Raises ImportException on error
		"""
		rec = self.__records[index.row()]
		
		# Ensure we record the same timestamp for this import
		self.__currentTimestamp = self.__currentTimestamp or QtCore.QDateTime.currentDateTime()

		query = QtSql.QSqlQuery()
		query.prepare("""
				INSERT INTO records
				(date, userid, accounttypeid, description, txdate, amount, insertdate, rawdata, checksum)
				VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
			""")

		query.addBindValue(rec.date)
		query.addBindValue(db.userId)
		query.addBindValue(accountId)
		query.addBindValue(rec.desc)
		query.addBindValue(rec.txdate)
		query.addBindValue(rec.credit or rec.debit)
		query.addBindValue(self.__currentTimestamp)
		query.addBindValue(QtCore.QString.fromUtf8(rec.data))
		query.addBindValue(rec.checksum)

		query.exec_()

		if query.lastError().isValid():
			raise ImportException(query.lastError().text())

		rec.imported = True
		self.dataSaved = True

		# Tell the view our data has changed
		self.dataChanged.emit(self.createIndex(index.row(), 0), self.createIndex(index.row(), self.columnCount() - 1))

	def reset(self):
		""" Slot to cancel all changes and revert to original records
		"""
		self.__records = deepcopy(self.__recordsRollback)
		self.dataSaved = False
		self.dataChanged.emit(self.createIndex(0, 0), self.createIndex(self.rowCount() -1, self.columnCount() - 1))
		self.__currentTimestamp = None
	
	def save(self):
		""" Slot to persist changes to records 
		"""
		self.__recordsRollback = deepcopy(self.__records)
		self.__currentTimestamp = None

	def loadRecords(self, records):
		""" Import the records into our model
			An input record is a tuple containing 
			(rawData, date, description, txDate, debit, credit, error,)

			The record is checked to see if it's already been imported into the database.
			For this we use the md5 checksum on the rawData field
		"""
		existingRecords = []
		query = QtSql.QSqlQuery('SELECT checksum from records where userid=%d' % db.userId)

		if query.lastError().isValid():
			raise ImportException(query.lastError().text())

		while query.next():
			existingRecords.append(query.value(0).toString())

		recordsLoaded = set()
		for record in set(records):
			rec = ImportRecord(*record)

			# Flag record as already imported
			if rec.valid and rec.checksum in existingRecords:
				rec.imported = True

			# Only import unique records or invalid ones (so we can see the error)  
			if not rec.valid or rec.checksum not in recordsLoaded:
				recordsLoaded.add(rec.checksum)
				self.__records.append(rec)

		# Take a copy of the records in case we want to rollback
		self.save()

	def canImport(self, index):
		""" Returns True if the record at index can be imported,
			ie no error and hasn't been imported yet
		"""
		rec = self.__records[index.row()]
		return not rec.imported and rec.valid

	def rowCount(self, parent=QtCore.QModelIndex()):
		return len(self.__records)

	@property
	def numBadRecords(self):
		""" Returns the number of records with csv import errors
		"""
		return len([rec for rec in self.__records if not rec.valid])

	@property
	def numRecordsImported(self):
		""" Returns the number of records that have already been imported
		"""
		return len([rec for rec in self.__records if rec.imported])

	@property
	def numRecordsToImport(self):
		""" Returns the number of records left that can be imported
		"""
		return len([rec for rec in self.__records if rec.valid and not rec.imported])

	def data(self, item, role=QtCore.Qt.DisplayRole):

		if not item.isValid():
			return QtCore.QVariant()

		if role == QtCore.Qt.BackgroundColorRole:
			if not self.__records[item.row()].valid:
				return QtGui.QColor("#ffd3cf")
			elif self.__records[item.row()].imported:
				return QtGui.QColor("#b6ffac")
			return QtCore.QVariant()

		if role == QtCore.Qt.ToolTipRole:
			return self.__records[item.row()].data 

		if role == QtCore.Qt.DisplayRole:
			if item.column() == 0:
				return self.__recordStatusToText(item)
			elif item.column() == 1:
				return self.__records[item.row()].date
			elif item.column() == 2:
				return self.__records[item.row()].txdate
			elif item.column() == 3:
				return self.__records[item.row()].desc
			elif item.column() == 4:
				return self.__records[item.row()].credit
			elif item.column() == 5:
				return self.__records[item.row()].debit

		return QtCore.QVariant()

	def columnCount(self, index=QtCore.QModelIndex()):
		""" Needs to match headerData
		"""
		return 6

	def headerData (self, section, orientation, role):
		if role == QtCore.Qt.DisplayRole:
			if section == 0:
				return "Status"
			elif section == 1:
				return "Date"
			elif section == 2:
				return "TxDate"
			elif section == 3:
				return "Description"
			elif section == 4:
				return "Credit"
			elif section == 5:
				return "Debit"
		return QtCore.QVariant()

	def __recordStatusToText(self, index):
		""" Returns the record status:
			"imported" if imported, or the error text or None
		"""
		rec = self.__records[index.row()]

		if not rec.valid:
			return rec.error
		elif rec.imported:
			return "Imported"
		else:
			return 'ready'

class RecordModel(QtSql.QSqlTableModel):
	def __init__(self, userId, parent=None):
		super(RecordModel, self).__init__(parent=parent)
		self.__userId = userId

	def select(self):
		status = super(RecordModel, self).select()
		while self.canFetchMore():
			self.fetchMore()

		return status

	def flags(self, index):
		flags = super(RecordModel, self).flags(index)
		if index.column() == enum.kRecordColumn_Checked:
			return flags | QtCore.Qt.ItemIsUserCheckable
		return flags


	def selectStatement(self):
		if self.tableName().isEmpty():
			return QtCore.QString()

		queryFilter = self.filter()
		queryFilter = 'AND ' + queryFilter if queryFilter else ''

		query = """
			SELECT
				r.recordid,
				r.checked,
				(select count(*) from recordtags rt where rt.recordid=r.recordid),
				r.checkdate,
				r.date,
				accounttypes.accountname,
				r.description,
				r.txdate,
				r.amount,
				r.insertdate,
				r.rawdata
			FROM records r
			INNER JOIN accounttypes ON accounttypes.accounttypeid=r.accounttypeid
			WHERE r.userid=%(userid)s
			%(filter)s
			ORDER BY r.date, r.description, r.txdate, r.recordid
		""" % {'userid': self.__userId, 'filter': queryFilter}

		return query


	def __getTagsforRecord(self, recordId):
		query = QtSql.QSqlQuery("""
			SELECT t.tagname
			FROM recordtags rt
			JOIN tags t ON t.tagid=rt.tagid
			WHERE rt.recordid=%d""" % recordId)

		tagNames = []
		while query.next():
			tagNames.append(str(query.value(0).toString()))

		return ', '.join(tagNames)

	def data(self, item, role=QtCore.Qt.DisplayRole):
		""" Return data from the model, formatted for viewing
		"""
		if not item.isValid():
			return QtCore.QVariant()

		if role == QtCore.Qt.CheckStateRole and item.column() == enum.kRecordColumn_Checked:
			if super(RecordModel, self).data(item).toBool():
				return QtCore.Qt.Checked
			else:
				return QtCore.Qt.Unchecked

		if role == QtCore.Qt.ToolTipRole:
			if item.column() == enum.kRecordColumn_Tags:
				#Show tag names for this record
				return self.__getTagsforRecord(self.data(self.index(item.row(), enum.kRecordColumn_RecordId)).toPyObject())

			elif item.column() == enum.kRecordColumn_Checked:
				if super(RecordModel, self).data(self.index(item.row(), enum.kRecordColumn_Checked)).toBool():
					return "Checked: " + super(RecordModel, self).data(
							self.index(item.row(), enum.kRecordColumn_CheckDate)).toDateTime().toString("dd/MM/yy hh:mm")

			elif item.column() == enum.kRecordColumn_Date:
				return "Imported: " + super(RecordModel, self).data(
							self.index(item.row(), enum.kRecordColumn_InsertDate)).toDateTime().toString("dd/MM/yy hh:mm")

			elif item.column() == enum.kRecordColumn_Description:
				return super(RecordModel, self).data(item).toString()

		if role == QtCore.Qt.UserRole and item.column() == enum.kRecordColumn_Tags:
			# Return the number of tags
			return super(RecordModel, self).data(item).toPyObject()

		if role == QtCore.Qt.BackgroundColorRole:
			# Indicate credit/debit with row colour
			val, ok = self.record(item.row()).value(enum.kRecordColumn_Amount).toDouble()
			if ok:
				return QtGui.QColor("#b6ffac") if val > 0.0 else QtGui.QColor("#ffd3cf")

		if role == QtCore.Qt.DecorationRole:
			if item.column() == enum.kRecordColumn_Tags:
				# Show tag icon if we have any
				if super(RecordModel, self).data(item).toPyObject() > 0:
					return QtGui.QIcon(':/icons/tag_yellow.png')


		if role == QtCore.Qt.DisplayRole:
			if item.column() in (enum.kRecordColumn_Checked, enum.kRecordColumn_Tags):
				# Don't display anything for these fields
				return QtCore.QVariant()

			elif item.column() == enum.kRecordColumn_Amount:
				# Display absolute currency values. credit/debit is indicated by background colour
				val, ok = super(RecordModel, self).data(item).toDouble()
				if ok:
					return QtCore.QString.number(abs(val), 'f', 2)

			elif item.column() == enum.kRecordColumn_Description:
				# Truncate the description field to 30 chars, and replace multiple spaces with single
				return super(RecordModel, self).data(item).toString().replace(QtCore.QRegExp('[ ]+'), ' ').left(30)

			elif item.column() == enum.kRecordColumn_Date:
				return super(RecordModel, self).data(item).toDate()

			elif item.column() == enum.kRecordColumn_Txdate:
				if super(RecordModel, self).data(item).toDateTime().time().toString() == "00:00:00":
					return super(RecordModel, self).data(item).toDate()
				else:
					return super(RecordModel, self).data(item).toDateTime()

		return super(RecordModel, self).data(item, role)

	def setData(self, index, value, role=QtCore.Qt.EditRole):
		""" Save new checkstate role changes in database
		"""
		if role == QtCore.Qt.CheckStateRole and index.column() == enum.kRecordColumn_Checked:
			if value.toPyObject() == QtCore.Qt.Checked:
				dbvalue = QtCore.QVariant(1)
				timestamp = QtCore.QVariant(QtCore.QDateTime.currentDateTime())
			else:
				dbvalue = QtCore.QVariant(0)
				timestamp = QtCore.QVariant()

			checkDateIndex = self.index(index.row(), enum.kRecordColumn_CheckDate)

			# Save the new state and timestamp
			if super(RecordModel, self).setData(index, dbvalue) and \
				super(RecordModel, self).setData(checkDateIndex, timestamp):
				return self.submitAll()

		return False

	def headerData (self, section, orientation, role=QtCore.Qt.DisplayRole):
		""" Set the header labels for the view
		"""
		if role == QtCore.Qt.DisplayRole:
			if section == enum.kRecordColumn_Checked:
				return "Check"
			elif section == enum.kRecordColumn_Tags:
				return "Tags"
			elif section == enum.kRecordColumn_Date:
				return "Date"
			elif section == enum.kRecordColumn_AccountType:
				return "Account"
			elif section == enum.kRecordColumn_Description:
				return "Description"
			elif section == enum.kRecordColumn_Txdate:
				return "TX date"
			elif section == enum.kRecordColumn_Amount:
				return "Amount"

		return QtCore.QVariant()

class TagModel(QtSql.QSqlTableModel):
	def __init__(self, recordIds, parent=None):
		super(TagModel, self).__init__(parent=parent)

		self.__recordIds = set(recordIds)
		self.setTable('tags')
		self.setEditStrategy(QtSql.QSqlTableModel.OnFieldChange)
		self.setFilter('userid=%d' % db.userId)
		self.select()

		model = QtSql.QSqlTableModel(self)
		model.setTable('recordtags')
		model.select()

		self.__recordTagModel = model

	def __contains__(self, tagName):
		""" Allows convenient:
				if "someTag" in model:
		"""
		for row in xrange(self.rowCount()):
			if tagName == self.index(row, enum.kTagsColumn_TagName).data().toString():
				return True
		return False

	def flags(self, index):
		return super(TagModel, self).flags(index) | QtCore.Qt.ItemIsUserCheckable

	def data(self, item, role=QtCore.Qt.DisplayRole):
		""" Tristate checkState:
				checked - all records have the tag
				partial - some records have the tag
				unchecked - no records have the tag
		"""
		if not item.isValid():
			return QtCore.QVariant()

		if role == QtCore.Qt.CheckStateRole and item.column() == enum.kTagsColumn_TagName:
			# Extract tagId for this tag
			tagId = super(TagModel, self).data(self.index(item.row(), enum.kTagsColumn_TagId)).toPyObject()
			recordIdsForTag = set(self.__getRecordIdsForTag(tagId))

			if self.__recordIds and self.__recordIds.issubset(recordIdsForTag):
				return QtCore.Qt.Checked
			elif self.__recordIds and self.__recordIds.intersection(recordIdsForTag):
				return QtCore.Qt.PartiallyChecked
			else:
				return QtCore.Qt.Unchecked

		return super(TagModel, self).data(item, role)

	def __getRecordIdsForTag(self, tagId):
		""" Generator to get all recordIds assigned to a given tag (id)
		"""
		for row in xrange(self.__recordTagModel.rowCount()):
			recordId = self.__recordTagModel.index(row, enum.kRecordTagsColumn_RecordId).data().toPyObject()
			recordTagId = self.__recordTagModel.index(row, enum.kRecordTagsColumn_TagId).data().toPyObject()

			if tagId == recordTagId:
				yield recordId

	def setData(self, index, value, role=QtCore.Qt.EditRole):
		""" Handle checkstate role changes 
		"""
		if role == QtCore.Qt.CheckStateRole:

			tagId = self.index(index.row(), enum.kTagsColumn_TagId).data().toPyObject()

			if value.toPyObject() == QtCore.Qt.Unchecked:
				self.__removeTagsFromRecords(tagId)
			else:
				self.__addTagsToRecords(tagId)

			self.dataChanged.emit(index, index)
			return True

		return super(TagModel, self).setData(index, value, role)

	def __removeTagsFromRecords(self, tagId):
		""" Delete a tag from our records
		"""
		for row in reversed(xrange(self.__recordTagModel.rowCount())):
			recordId = self.__recordTagModel.index(row, enum.kRecordTagsColumn_RecordId).data().toPyObject()
			if recordId not in self.__recordIds:
				continue

			recordTagId = self.__recordTagModel.index(row, enum.kRecordTagsColumn_TagId).data().toPyObject()
			if recordTagId != tagId:
				continue

			self.__recordTagModel.removeRows(row, 1)
			self.__recordTagModel.submit()


	def __addTagsToRecords(self, tagId):
		""" Assign the tag to our records
		"""
		currentIds = set(self.__getRecordIdsForTag(tagId))
		newRecordIds = self.__recordIds - currentIds

		row = self.__recordTagModel.rowCount()
		self.__recordTagModel.insertRows(row, 1)

		for recordId in newRecordIds:
			# Only one row at a time can be inserted when using the OnFieldChange or OnRowChange update strategies.
			self.__recordTagModel.insertRows(row, 1)
			self.__recordTagModel.setData(self.__recordTagModel.index(row, enum.kRecordTagsColumn_RecordId), QtCore.QVariant(recordId))
			self.__recordTagModel.setData(self.__recordTagModel.index(row, enum.kRecordTagsColumn_TagId), QtCore.QVariant(tagId))

			self.__recordTagModel.submit()


	def addTag(self, tagName):
		""" Add a new tag and assign to our current records
		"""
		rowCount = self.rowCount()
		self.insertRows(rowCount, 1)
		self.setData(self.index(rowCount, enum.kTagsColumn_TagName), QtCore.QVariant(tagName))
		self.setData(self.index(rowCount, enum.kTagsColumn_UserId), QtCore.QVariant(db.userId))
		self.submit()

		tagId = self.data(self.index(rowCount, enum.kTagsColumn_TagId)).toPyObject()
		self.__addTagsToRecords(tagId)


class SortProxyModel(QtGui.QSortFilterProxyModel):
	def __init__(self, parent=None):
		super(SortProxyModel, self).__init__(parent=parent)

	def lessThan(self, left, right):
		""" Define the comparison to ensure column data is sorted correctly
		"""
		if left.column() == enum.kRecordColumn_Tags:
			return self.sourceModel().data(left, QtCore.Qt.UserRole) < self.sourceModel().data(right, QtCore.Qt.UserRole)

		elif left.column() == enum.kRecordColumn_Checked:
			return self.sourceModel().data(left, QtCore.Qt.CheckStateRole) < self.sourceModel().data(right, QtCore.Qt.CheckStateRole)

		elif left.column() == enum.kRecordColumn_Amount:
			leftVal, leftOk = self.sourceModel().record(left.row()).value(enum.kRecordColumn_Amount).toDouble()
			rightVal, rightOk = self.sourceModel().record(right.row()).value(enum.kRecordColumn_Amount).toDouble()
			if leftOk and rightOk:
				return leftVal < rightVal

		return super(SortProxyModel, self).lessThan(left, right)


class AccountModel(QtSql.QSqlTableModel):
	def __init__(self, parent=None):
		super(AccountModel, self).__init__(parent=parent)

	def data(self, item, role=QtCore.Qt.DisplayRole):
		if not item.isValid():
			return QtCore.QVariant()

		if role == QtCore.Qt.BackgroundColorRole and self.isDirty(item):
				return QtGui.QColor(255,156,126)

		return super(AccountModel, self).data(item, role)

	def setData(self, index, value, role=QtCore.Qt.EditRole):
		# Don't flag cell as changed when it hasn't
		if role == QtCore.Qt.EditRole and self.data(index, role) == value:
			return False

		return super(AccountModel, self).setData(index, value, role)

	def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):

		if role == QtCore.Qt.DisplayRole:
			if section == enum.kAccountTypeColumn_AccountName:
				return 'Account Name'
			elif section == enum.kAccountTypeColumn_DateField:
				return 'Date'
			elif section == enum.kAccountTypeColumn_DescriptionField:
				return 'Desc'
			elif section == enum.kAccountTypeColumn_CreditField:
				return 'Credit'
			elif section == enum.kAccountTypeColumn_DebitField:
				return 'Debit'
			elif section == enum.kAccountTypeColumn_CurrencySign:
				return 'Currency sign'
			elif section == enum.kAccountTypeColumn_DateFormat:
				return 'Date formats'

		return QtCore.QVariant()

class CheckComboModel(QtSql.QSqlTableModel):

	checkStateChanged = QtCore.pyqtSignal()
	dataChanged = QtCore.pyqtSignal('QModelIndex, QModelIndex)')

	def __init__(self, parent=None):
		super(CheckComboModel, self).__init__(parent=parent)
		self._checkedItems = set()
		self.__userRoleColumn = None

	def setUserRoleColumn(self, column):
		""" Set the database table colum to use for UserRole data
		"""
		self.__userRoleColumn = column

	def flags(self, index):
		return super(CheckComboModel, self).flags(index) | QtCore.Qt.ItemIsUserCheckable

	def data(self, index, role):
		if not index.isValid():
			return QtCore.QVariant()

		if role == QtCore.Qt.CheckStateRole:
			if index.row() in self._checkedItems:
				return QtCore.Qt.Checked
			else:
				return QtCore.Qt.Unchecked

		if role == QtCore.Qt.UserRole and self.__userRoleColumn is not None:
			return self.record(index.row()).value(self.__userRoleColumn).toPyObject()

		return super(CheckComboModel, self).data(index, role)

	def setData(self, index, value, role):
		if role == QtCore.Qt.CheckStateRole:

			if value.toPyObject() == QtCore.Qt.Checked:
				self._checkedItems.add(index.row())
			else:
				self._checkedItems.remove(index.row())

			self.emit(QtCore.SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)
			self.emit(QtCore.SIGNAL('checkStateChanged()'))

			return True

		return False
